#![allow(unused_imports)]

use enumflags2::{bitflags, make_bitflags, BitFlags};

use crate::memory::MemoryRegion;

use crate::memory::alloc;
use alloc::string::String;
use alloc::vec::Vec;

#[allow(dead_code)]
pub struct ProcessTable {}

#[allow(dead_code)]
pub struct Process {
    /// Unique process ID.
    proc_id: u32,
    /// ID of the parent process.
    parent_proc_id: u32,
    /// List of all child processes.
    child_proc_ids: Vec<u32>,
    /// Human readable name of the process, should be the command.
    name: String,
    /// The base address of the process stack within its memory region.
    stack_base_addr: usize,
    /// The length of the stack. This, plus the start address, is the stack size.
    stack_length: usize,
    /// The stack pointer, which is relative to the memory region start.
    stack_ptr: usize,
    /// The region of memory allocated to the process.
    mem_region: MemoryRegion,
    /// Process priority. Lower number is higher priority.
    prio: u16,
    /// Which semaphor the process is waiting on.
    semaphor_wait: Option<u32>,
    /// What capabilities the process has
    capabilities: BitFlags<ProcessCapabilities>,
}

#[allow(dead_code)]
pub unsafe fn context_switch() -> ! {
    #[allow(unused_unsafe)]
    unsafe {
        loop {
            crate::arch::current::asm::halt();
        }
    }
}

#[allow(dead_code)]
pub enum ProcessState {
    Running,
    Waiting,
    Sleeping,
    Suspended,
}

#[bitflags]
#[repr(u32)]
#[derive(Copy, Clone)]
pub enum ProcessCapabilities {
    // Process capabilities
    ProcessEnum, // Enumerate running process IDs and names
    ProcessRead, // Read information from a process struct
    ProcessKill, // Kill any process
    ProcessSpawn, // Create a new process
    ProcessExec, // Replace self with new process image
    // File system capabilities
    FileEnum, // Enumerate directories and files
    FileRead, // Read files
    FileWrite, // Write to files
    FilePermission, // Modify file permissions
    FileCreate, // Create files
    FileDelete, // Delete files
    FileSystem, // Mount, unmount, and modify filesystems
    // Kernel config capabilities
    KernelCfgRead, // Read kernel configurations
    KernelCfgWrite, // Modify kernel configurations
}
